#!/bin/env python

import argparse
import os
import subprocess
import sys
import urllib


def do_command(server_auth, cmd, inputf=None, outputf=None):
    infile = None
    outfile = None
    cmd_str = "gpg-proxy-sign " + ' '.join(cmd)

    if inputf:
        infile = open(inputf)
    if outputf:
        outfile = open(outputf, 'w')
    proc = subprocess.Popen(['ssh', server_auth, cmd_str], stdin=infile,
                            stdout=outfile)
    proc.communicate()
    if proc.returncode != 0:
        raise RuntimeError("Failed to do command")

    if infile:
        infile.close()
    if outfile:
        outfile.close()


def export_pubkey(server_auth, args):
    """
    Request the server to export a public key
    """
    cmd_args = ['--armor'] if args.armor else ['--no-armor']
    cmd = ['--export'] + cmd_args + ['--local-user', urllib.quote(args.input[0])]
    do_command(server_auth, cmd, outputf=args.output)


def detach_sign(server_auth, args):
    """
    Request the server create a detached signature
    """
    output = args.output
    if output is None:
        suffix = '.asc' if args.armor else '.sig'
        output = args.input[0] + suffix

    cmd_args = ['--local-user', urllib.quote(args.local_user)]
    cmd_args.append('--armor' if args.armor else '--no-armor')
    cmd = ['--detach-sign'] + cmd_args
    do_command(server_auth, cmd, inputf=args.input[0], outputf=output)


def parse_args_or_exit(argv=None):
    """
    Parse command line options
    """
    parser = argparse.ArgumentParser(description="GnuPG signing proxy client")
    parser.add_argument('input', nargs='*')
    parser.add_argument("--output", "-o", default=None)
    parser.add_argument("--armor", action='store_true')
    parser.add_argument("--no-armor", action='store_false', dest='armor')
    parser.add_argument("--export", action='store_const', const='export',
                        dest='action')
    parser.add_argument("--local-user", "--default-key", "-u", default=None)
    parser.add_argument("--sign", "-s", action='store_const', const='sign',
                        dest='action')
    parser.add_argument("--detach-sign", "-b", action='store_const',
                        const='detach_sign', dest='action')

    # ignored
    parser.add_argument("--batch", action='store_true')
    parser.add_argument("--no-verbose", action='store_true')
    parser.add_argument("--passphrase-fd", type=int)
    parser.add_argument("--no-secmem-warning", action='store_true')
    parser.add_argument("--yes", action='store_true')

    return parser.parse_args(argv)


def main(argv):
    """
    Pass GnuPG operations to remote server
    """
    args = parse_args_or_exit(argv)
    gpg_server = os.environ.get('UPDATE_GPG_SERVER', 'localhost')
    gpg_user = os.environ.get('UPDATE_GPG_USER', 'sign')
    server_auth = "%s@%s" % (gpg_user, gpg_server)

    if args.action == "export":
        export_pubkey(server_auth, args)
    elif args.action == "sign":
        if args.output != '-':
            raise RuntimeError("Non-detached signing not implemented")
    elif args.action == "detach_sign":
        detach_sign(server_auth, args)
    else:
        raise RuntimeError("Unimplemented action: " + str(args.action))


def _main():
    """
    Entry point for setuptools CLI wrapper
    """
    main(sys.argv[1:])


# Entry point when run directly
if __name__ == "__main__":
    _main()
