import argparse
import subprocess
import sys


def key_hash(output):
    """
    Extract the hash of a key from gpg output
    """
    output_list = output.split('\n')
    details = output_list[0].split(None, 3)
    _, khash = details[1].split('/', 1)
    return khash


def add_key(keyfile):
    """
    Import a public key and trust it
    """
    output = subprocess.check_output(['gpg', '--with-fingerprint', keyfile])
    khash = key_hash(output)
    subprocess.check_call(['gpg', '--import', keyfile])
    proc = subprocess.Popen(['gpg', '--command-fd', '0', '--edit-key', khash],
                            stdin=subprocess.PIPE)
    proc.communicate("trust\n5\ny\n")
    if proc.returncode != 0:
        raise RuntimeError("Failed to set trust on key")


def remove_key(keyname):
    """
    Remove a public key by name
    """
    output = subprocess.check_output(['gpg', '-k', keyname])
    khash = key_hash(output)
    subprocess.check_call(['gpg', '--batch', '--yes', '--delete-key', khash])


def parse_args_or_exit(argv=None):
    """
    Parse command line options
    """
    parser = argparse.ArgumentParser(description="Add or remove authorizing public keys")
    parser.add_argument("--add", action='append',
                        help="File name of gpg key to add")
    parser.add_argument("--remove", "--delete", action='append',
                        help="Name of gpg key to remove")

    return parser.parse_args(argv)


def main(argv):
    """
    Manage public keys that may sign requests
    """
    args = parse_args_or_exit(argv)

    if args.add:
        for keyfile in args.add:
            add_key(keyfile)

    if args.remove:
        for keyname in args.remove:
            remove_key(keyname)


def _main():
    """
    Entry point for setuptools CLI wrapper
    """
    main(sys.argv[1:])


# Entry point when run directly
if __name__ == "__main__":
    _main()
