#!/usr/env python

import argparse
import ConfigParser
import glob
import os
import os.path
import shutil
import subprocess
import sys
import tempfile
import uuid

from update_package.common import *


def unpack_iso(iso_name):
    """
    Unpack ISO image into working directory
    """
    work_dir = tempfile.mkdtemp(prefix='sign-media-')
    unpack_dir = os.path.join(work_dir, 'root')
    os.mkdir(unpack_dir)
    unpack_archive(iso_name, unpack_dir)
    return work_dir


def add_keys(work_dir, keyfiles):
    """
    Add public keys to media
    """
    keynum = 1
    root_dir = os.path.join(work_dir, 'root')
    conffile = os.path.join(root_dir, '.treeinfo')
    treeinfo = ConfigParser.SafeConfigParser()
    treeinfo.read([conffile])
    treeinfo.remove_section('keys')
    treeinfo.add_section('keys')
    oldfiles = glob.glob(os.path.join(root_dir, 'RPM-GPG-*'))
    for oldfile in oldfiles:
        os.remove(oldfile)
    for keyfile in keyfiles:
        basefile = os.path.basename(keyfile)
        copy_remote_file(keyfile, os.path.join(root_dir, basefile))
        treeinfo.set('keys', "key%d" % keynum, basefile)
        keynum += 1
    with open(conffile, 'w') as fileh:
        treeinfo.write(fileh)


def parse_args_or_exit(argv=None):
    """
    Parse command line options
    """
    parser = argparse.ArgumentParser(description="Sign update media")
    parser.add_argument("-i", "--input", metavar="ISO", required=True,
                        help="Input ISO file")
    parser.add_argument("-o", "--output", metavar="ISO", required=True,
                        help="Output ISO file")
    parser.add_argument("-k", "--key", metavar="UID",
                        help="UID of gpg key")
    parser.add_argument("--keyfile", required=True, action='append',
                        help="File name of gpg key")
    parser.add_argument('--no-passphrase', action='store_false',
                        dest="prompt_pass", help="Disable passphrase prompt")

    return parser.parse_args(argv)


def main(argv):
    """
    Unpack a media ISO, (re)sign constituent parts then reassemble
    """
    args = parse_args_or_exit(argv)
    passphrase = None
    if args.key and args.prompt_pass:
        passphrase = get_gpg_key_pass(args.key)
    media = fetch_remote_file(args.input)
    print "Unpacking media image..."
    work_dir = unpack_iso(media)
    try:
        uuid_str, label = read_volume_desc(media)
        if uuid_str is None:
            uuid_str = str(uuid.uuid4())
            print "Generated new UUID:", uuid_str
        if args.key:
            packages = glob.glob(os.path.join(work_dir, "root/Packages/*.rpm"))
            packages += glob.glob(os.path.join(work_dir,
                                               "root/client_install/*.rpm"))
            sign_packages(work_dir, args.key, passphrase, packages)
            group_file = copy_group_file(work_dir)
            shutil.rmtree(os.path.join(work_dir, 'root/repodata'),
                          ignore_errors=True)
            create_repo(work_dir, args.key, passphrase, group_file, 'Packages')

        add_keys(work_dir, args.keyfile)
        build_bootable_iso(work_dir, args.output, uuid_str, label)
        print "Wrote media image to %s with UUID %s" % (args.output, uuid_str)
    finally:
        shutil.rmtree(work_dir, ignore_errors=True)
        if media != args.input:
            shutil.rmtree(os.path.dirname(media), ignore_errors=True)


def _main():
    """
    Entry point for setuptools CLI wrapper
    """
    main(sys.argv[1:])


# Entry point when run directly
if __name__ == "__main__":
    _main()
