#!/usr/env python

import argparse
import glob
import os
import os.path
import shutil
import subprocess
import sys
import tempfile
import uuid

import update_package.common


def unpack_iso(iso_name):
    """
    Unpack ISO image into working directory
    """
    work_dir = tempfile.mkdtemp(prefix='sign-source-')
    unpack_dir = os.path.join(work_dir, 'root')
    os.mkdir(unpack_dir)
    update_package.common.unpack_archive(iso_name, unpack_dir)
    return work_dir


def add_keys(work_dir, keyfiles):
    """
    Add public keys to media
    """
    for keyfile in keyfiles:
        basefile = os.path.basename(keyfile)
        shutil.copyfile(keyfile, os.path.join(work_dir, 'root', basefile))


def parse_args_or_exit(argv=None):
    """
    Parse command line options
    """
    parser = argparse.ArgumentParser(description="Sign source image")
    parser.add_argument("-i", "--input", metavar="ISO", required=True,
                        help="Input ISO file")
    parser.add_argument("-o", "--output", metavar="ISO", required=True,
                        help="Output ISO file")
    parser.add_argument("-k", "--key", metavar="UID", required=True,
                        help="UID of gpg key")
    parser.add_argument("--keyfile", required=True, action='append',
                        help="File name of gpg key")
    parser.add_argument('--no-passphrase', action='store_false',
                        dest="prompt_pass", help="Disable passphrase prompt")

    return parser.parse_args(argv)


def main(argv):
    """
    Unpack a source ISO, (re)sign constituent parts then reassemble
    """
    args = parse_args_or_exit(argv)
    passphrase = None
    if args.prompt_pass:
        passphrase = get_gpg_key_pass(args.key)
    source = update_package.common.fetch_remote_file(args.input)
    print "Unpacking source image..."
    work_dir = unpack_iso(source)
    try:
        uuid_str, label = update_package.common.read_volume_desc(source)
        if uuid_str is None:
            uuid_str = str(uuid.uuid4())
            print "Generated new UUID:", uuid_str
        packages = glob.glob(os.path.join(work_dir, "root/*.rpm"))
        packages += glob.glob(os.path.join(work_dir, "root/*/*.rpm"))
        if packages:
            update_package.common.sign_packages(work_dir, args.key, passphrase,
                                                packages)
        group_file = update_package.common.copy_group_file(work_dir)
        shutil.rmtree(os.path.join(work_dir, 'root/repodata'),
                      ignore_errors=True)
        update_package.common.create_repo(work_dir, args.key, passphrase,
                                          group_file)
        add_keys(work_dir, args.keyfile)
        update_package.common.build_iso(work_dir, args.output, uuid_str, label)
        print "Wrote source image to %s with UUID %s" % (args.output, uuid_str)
    finally:
        shutil.rmtree(work_dir, ignore_errors=True)
        if source != args.input:
            shutil.rmtree(os.path.dirname(source), ignore_errors=True)


def _main():
    """
    Entry point for setuptools CLI wrapper
    """
    main(sys.argv[1:])


# Entry point when run directly
if __name__ == "__main__":
    _main()
