#!/usr/env python

import argparse
import glob
import os
import os.path
import shutil
import subprocess
import sys
import tempfile
import xml.dom.minidom

from update_package.common import *


def unpack_iso(iso_name):
    """
    Unpack ISO image into working directory
    """
    work_dir = tempfile.mkdtemp(prefix='sign-update-')
    unpack_dir = os.path.join(work_dir, 'root')
    os.mkdir(unpack_dir)
    unpack_archive(iso_name, unpack_dir)
    return work_dir


def edit_update_doc(work_dir, keyfile):
    """
    Replace public key file in update document
    """
    filename = os.path.join(work_dir, 'root/update.xml')
    doc = xml.dom.minidom.parse(filename)
    top = doc.documentElement
    top.setAttribute('key', os.path.basename(keyfile))
    with open(filename, "w") as fileh:
        top.writexml(fileh, addindent="  ", newl='\n')


def sign_update(iso_name, args, passphrase):
    """
    Unpack a single update and sign
    """
    work_dir = unpack_iso(iso_name)
    try:
        uuid, label = read_update_doc(
            os.path.join(work_dir, 'root/update.xml'))
        if args.keyfile:
            edit_update_doc(work_dir, args.keyfile)
        sign_update_doc(work_dir, args.key, passphrase)
        packages = glob.glob(os.path.join(work_dir, "root/Packages/*.rpm"))
        sign_packages(work_dir, args.key, passphrase,
                                            packages)
        group_file = copy_group_file(work_dir)
        shutil.rmtree(os.path.join(work_dir, 'root/repodata'),
                      ignore_errors=True)
        create_repo(work_dir, args.key, passphrase,
                                          group_file)
        outfile = os.path.join(args.output, os.path.basename(iso_name))
        build_iso(work_dir, outfile, uuid, label)
    finally:
        shutil.rmtree(work_dir, ignore_errors=True)


def parse_args_or_exit(argv=None):
    """
    Parse command line options
    """
    parser = argparse.ArgumentParser(description="Sign update media")
    parser.add_argument("input", metavar="ISO", nargs="+",
                        help="Input ISO file")
    parser.add_argument("-o", "--output", metavar="DIR", required=True,
                        help="Output directory")
    parser.add_argument("-k", "--key", metavar="UID", required=True,
                        help="UID of gpg key")
    parser.add_argument("--keyfile", default=None,
                        help="File name of gpg key")
    parser.add_argument('--no-passphrase', action='store_false',
                        dest="prompt_pass", help="Disable passphrase prompt")

    return parser.parse_args(argv)


def main(argv):
    """
    Unpack multiple update ISOs, sign constituent parts then reassemble
    """
    args = parse_args_or_exit(argv)
    passphrase = None
    if args.prompt_pass:
        passphrase = get_gpg_key_pass(args.key)
    for iso in args.input:
        local_iso = fetch_remote_file(iso)
        sign_update(local_iso, args, passphrase)
        if local_iso != iso:
            shutil.rmtree(os.path.dirname(local_iso), ignore_errors=True)


def _main():
    """
    Entry point for setuptools CLI wrapper
    """
    main(sys.argv[1:])


# Entry point when run directly
if __name__ == "__main__":
    _main()
